@@ -28,11 +28,11 @@ class HTMLExporter : public AbstractExporter
 {
 public:
     HTMLExporter(KTextEditor::View *view, QTextStream &output, const bool withHeaderFooter = false);
-    virtual ~HTMLExporter();
+    ~HTMLExporter() override;
 
-    virtual void openLine();
-    virtual void closeLine(const bool lastLine);
-    virtual void exportText(const QString &text, const KTextEditor::Attribute::Ptr &attrib);
+    void openLine() override;
+    void closeLine(const bool lastLine) override;
+    void exportText(const QString &text, const KTextEditor::Attribute::Ptr &attrib) override;
 };
 
 #endif
