@@ -24,6 +24,7 @@
 
 #include <ktexteditor/view.h>
 #include <ktexteditor/texthintinterface.h>
+#include <ktexteditor/inlinenoteinterface.h>
 #include <ktexteditor/markinterface.h>
 #include <ktexteditor/codecompletioninterface.h>
 #include <ktexteditor/configinterface.h>
@@ -46,6 +47,7 @@ namespace KTextEditor
 {
 class AnnotationModel;
 class Message;
+class InlineNoteProvider;
 }
 
 namespace KTextEditor { class DocumentPrivate; }
@@ -68,6 +70,7 @@ class KateModeMenu;
 class KateAbstractInputMode;
 class KateScriptActionMenu;
 class KateMessageLayout;
+class KateInlineNoteData;
 
 class KToggleAction;
 class KSelectAction;
@@ -84,13 +87,16 @@ class KTEXTEDITOR_EXPORT ViewPrivate : public KTextEditor::View,
     public KTextEditor::TextHintInterface,
     public KTextEditor::CodeCompletionInterface,
     public KTextEditor::ConfigInterface,
-    public KTextEditor::AnnotationViewInterface
+    public KTextEditor::InlineNoteInterface,
+    public KTextEditor::AnnotationViewInterfaceV2
 {
     Q_OBJECT
     Q_INTERFACES(KTextEditor::TextHintInterface)
     Q_INTERFACES(KTextEditor::ConfigInterface)
     Q_INTERFACES(KTextEditor::CodeCompletionInterface)
     Q_INTERFACES(KTextEditor::AnnotationViewInterface)
+    Q_INTERFACES(KTextEditor::AnnotationViewInterfaceV2)
+    Q_INTERFACES(KTextEditor::InlineNoteInterface)
 
     friend class KTextEditor::View;
     friend class ::KateViewInternal;
@@ -99,23 +105,23 @@ class KTEXTEDITOR_EXPORT ViewPrivate : public KTextEditor::View,
 
 public:
     ViewPrivate (KTextEditor::DocumentPrivate *doc, QWidget *parent, KTextEditor::MainWindow *mainWindow = nullptr);
-    ~ViewPrivate ();
+    ~ViewPrivate() override;
 
     /**
      * Get the view's main window, if any
      * \return the view's main window
      */
-    KTextEditor::MainWindow *mainWindow() const Q_DECL_OVERRIDE
+    KTextEditor::MainWindow *mainWindow() const override
     {
         return m_mainWindow;
     }
 
-    KTextEditor::Document *document() const Q_DECL_OVERRIDE;
+    KTextEditor::Document *document() const override;
 
-    ViewMode viewMode() const Q_DECL_OVERRIDE;
-    QString viewModeHuman() const Q_DECL_OVERRIDE;
-    InputMode viewInputMode() const Q_DECL_OVERRIDE;
-    QString viewInputModeHuman() const Q_DECL_OVERRIDE;
+    ViewMode viewMode() const override;
+    QString viewModeHuman() const override;
+    InputMode viewInputMode() const override;
+    QString viewInputModeHuman() const override;
 
     void setInputMode(InputMode mode);
 
@@ -129,7 +135,8 @@ public Q_SLOTS:
 
 private Q_SLOTS:
     /**
-     * internal use, apply word wrap
+     * Wrap lines touched by the selection with respect of existing paragraphs.
+     * Work is done by KTextEditor::DocumentPrivate::wrapParagraph
      */
     void applyWordWrap();
 
@@ -137,9 +144,9 @@ private Q_SLOTS:
     // KTextEditor::PopupMenuInterface
     //
 public:
-    void setContextMenu(QMenu *menu) Q_DECL_OVERRIDE;
-    QMenu *contextMenu() const Q_DECL_OVERRIDE;
-    QMenu *defaultContextMenu(QMenu *menu = nullptr) const Q_DECL_OVERRIDE;
+    void setContextMenu(QMenu *menu) override;
+    QMenu *contextMenu() const override;
+    QMenu *defaultContextMenu(QMenu *menu = nullptr) const override;
 
 private Q_SLOTS:
     void aboutToShowContextMenu();
@@ -152,17 +159,17 @@ private:
     // KTextEditor::ViewCursorInterface
     //
 public:
-    bool setCursorPosition(KTextEditor::Cursor position) Q_DECL_OVERRIDE;
+    bool setCursorPosition(KTextEditor::Cursor position) override;
 
-    KTextEditor::Cursor cursorPosition() const Q_DECL_OVERRIDE;
+    KTextEditor::Cursor cursorPosition() const override;
 
-    KTextEditor::Cursor cursorPositionVirtual() const Q_DECL_OVERRIDE;
+    KTextEditor::Cursor cursorPositionVirtual() const override;
 
-    QPoint cursorToCoordinate(const KTextEditor::Cursor &cursor) const Q_DECL_OVERRIDE;
+    QPoint cursorToCoordinate(const KTextEditor::Cursor &cursor) const override;
 
-    KTextEditor::Cursor coordinatesToCursor(const QPoint &coord) const Q_DECL_OVERRIDE;
+    KTextEditor::Cursor coordinatesToCursor(const QPoint &coord) const override;
 
-    QPoint cursorPositionCoordinates() const Q_DECL_OVERRIDE;
+    QPoint cursorPositionCoordinates() const override;
 
     bool setCursorPositionVisual(const KTextEditor::Cursor &position);
 
@@ -173,8 +180,8 @@ public:
      */
     int virtualCursorColumn() const;
 
-    bool mouseTrackingEnabled() const Q_DECL_OVERRIDE;
-    bool setMouseTrackingEnabled(bool enable) Q_DECL_OVERRIDE;
+    bool mouseTrackingEnabled() const override;
+    bool setMouseTrackingEnabled(bool enable) override;
 
 private:
     void notifyMousePositionChanged(const KTextEditor::Cursor &newPosition);
@@ -187,40 +194,55 @@ public:
     // KTextEditor::ConfigInterface
     //
 public:
-    QStringList configKeys() const Q_DECL_OVERRIDE;
-    QVariant configValue(const QString &key) Q_DECL_OVERRIDE;
-    void setConfigValue(const QString &key, const QVariant &value) Q_DECL_OVERRIDE;
+    QStringList configKeys() const override;
+    QVariant configValue(const QString &key) override;
+    void setConfigValue(const QString &key, const QVariant &value) override;
 
 Q_SIGNALS:
     void configChanged();
 
 public:
     /**
-     * Try to fold starting at the given line.
-     * This will both try to fold existing folding ranges of this line and to query the highlighting what to fold.
-     * @param startLine start line to fold at
+     * Try to fold an unfolded range starting at @p line
+     * @return the new folded range on success, otherwise an unvalid range
      */
-    void foldLine(int startLine);
+    KTextEditor::Range foldLine(int line);
 
     /**
-    * Try to unfold all foldings starting at the given line.
-    * @param startLine start line to unfold at
-    */
-    void unfoldLine(int startLine);
+     * Try to unfold a folded range starting at @p line
+     * @return true when a range was unfolded, otherwise false
+     */
+    bool unfoldLine(int line);
+
+    /**
+     * Try to toggle the folding state of a range starting at line @p line
+     * @return true when the line was toggled, false when not
+     */
+    bool toggleFoldingOfLine(int line);
+
+    /**
+     * Try to change all the foldings inside a folding range starting at @p line
+     * but not the range itself starting there.
+     * However, should the range itself be folded, will only the range unfolded
+     * and the containing ranges kept untouched.
+     * Should the range not contain other ranges will the range itself folded,
+     * @return true when any range was folded or unfolded, otherwise false
+     */
+    bool toggleFoldingsInRange(int line);
 
     //
     // KTextEditor::CodeCompletionInterface2
     //
 public:
-    bool isCompletionActive() const Q_DECL_OVERRIDE;
-    void startCompletion(const KTextEditor::Range &word, KTextEditor::CodeCompletionModel *model) Q_DECL_OVERRIDE;
-    void abortCompletion() Q_DECL_OVERRIDE;
-    void forceCompletion() Q_DECL_OVERRIDE;
-    void registerCompletionModel(KTextEditor::CodeCompletionModel *model) Q_DECL_OVERRIDE;
-    void unregisterCompletionModel(KTextEditor::CodeCompletionModel *model) Q_DECL_OVERRIDE;
+    bool isCompletionActive() const override;
+    void startCompletion(const KTextEditor::Range &word, KTextEditor::CodeCompletionModel *model) override;
+    void abortCompletion() override;
+    void forceCompletion() override;
+    void registerCompletionModel(KTextEditor::CodeCompletionModel *model) override;
+    void unregisterCompletionModel(KTextEditor::CodeCompletionModel *model) override;
     bool isCompletionModelRegistered(KTextEditor::CodeCompletionModel *model) const;
-    bool isAutomaticInvocationEnabled() const Q_DECL_OVERRIDE;
-    void setAutomaticInvocationEnabled(bool enabled = true) Q_DECL_OVERRIDE;
+    bool isAutomaticInvocationEnabled() const override;
+    void setAutomaticInvocationEnabled(bool enabled = true) override;
 
 Q_SIGNALS:
     void completionExecuted(KTextEditor::View *view, const KTextEditor::Cursor &position, KTextEditor::CodeCompletionModel *model, const QModelIndex &);
@@ -239,10 +261,10 @@ public:
     // KTextEditor::TextHintInterface
     //
 public:
-    void registerTextHintProvider(KTextEditor::TextHintProvider *provider) Q_DECL_OVERRIDE;
-    void unregisterTextHintProvider(KTextEditor::TextHintProvider *provider) Q_DECL_OVERRIDE;
-    void setTextHintDelay(int delay) Q_DECL_OVERRIDE;
-    int textHintDelay() const Q_DECL_OVERRIDE;
+    void registerTextHintProvider(KTextEditor::TextHintProvider *provider) override;
+    void unregisterTextHintProvider(KTextEditor::TextHintProvider *provider) override;
+    void setTextHintDelay(int delay) override;
+    int textHintDelay() const override;
 
 public:
     bool dynWordWrap() const
@@ -250,23 +272,40 @@ public:
         return m_hasWrap;
     }
 
+    //
+    // Inline Notes Interface
+    //
+public:
+    void registerInlineNoteProvider(KTextEditor::InlineNoteProvider *provider) override;
+    void unregisterInlineNoteProvider(KTextEditor::InlineNoteProvider *provider) override;
+    QRect inlineNoteRect(const KateInlineNoteData& note) const;
+
+    QVarLengthArray<KateInlineNoteData, 8> inlineNotes(int line) const;
+
+private:
+    QVector<KTextEditor::InlineNoteProvider *> m_inlineNoteProviders;
+
+private Q_SLOTS:
+    void inlineNotesReset();
+    void inlineNotesLineChanged(int line);
+
     //
     // KTextEditor::SelectionInterface stuff
     //
 public Q_SLOTS:
-    bool setSelection(const KTextEditor::Range &selection) Q_DECL_OVERRIDE;
+    bool setSelection(const KTextEditor::Range &selection) override;
 
-    bool removeSelection() Q_DECL_OVERRIDE
+    bool removeSelection() override
     {
         return clearSelection();
     }
 
-    bool removeSelectionText() Q_DECL_OVERRIDE
+    bool removeSelectionText() override
     {
         return removeSelectedText();
     }
 
-    bool setBlockSelection(bool on) Q_DECL_OVERRIDE;
+    bool setBlockSelection(bool on) override;
     bool toggleBlockSelection();
 
     bool clearSelection();
@@ -277,10 +316,10 @@ public Q_SLOTS:
     bool selectAll();
 
 public:
-    bool selection() const Q_DECL_OVERRIDE;
-    QString selectionText() const Q_DECL_OVERRIDE;
-    bool blockSelection() const Q_DECL_OVERRIDE;
-    KTextEditor::Range selectionRange() const Q_DECL_OVERRIDE;
+    bool selection() const override;
+    QString selectionText() const override;
+    bool blockSelection() const override;
+    KTextEditor::Range selectionRange() const override;
 
     static void blockFix(KTextEditor::Range &range);
 
@@ -343,16 +382,20 @@ public:
     // KTextEditor::AnnotationView
     //
 public:
-    void setAnnotationModel(KTextEditor::AnnotationModel *model) Q_DECL_OVERRIDE;
-    KTextEditor::AnnotationModel *annotationModel() const Q_DECL_OVERRIDE;
-    void setAnnotationBorderVisible(bool visible) Q_DECL_OVERRIDE;
-    bool isAnnotationBorderVisible() const Q_DECL_OVERRIDE;
+    void setAnnotationModel(KTextEditor::AnnotationModel *model) override;
+    KTextEditor::AnnotationModel *annotationModel() const override;
+    void setAnnotationBorderVisible(bool visible) override;
+    bool isAnnotationBorderVisible() const override;
+    void setAnnotationItemDelegate(KTextEditor::AbstractAnnotationItemDelegate *delegate) override;
+    KTextEditor::AbstractAnnotationItemDelegate* annotationItemDelegate() const override;
+    void setAnnotationUniformItemSizes(bool enable) override;
+    bool uniformAnnotationItemSizes() const override;
 
 Q_SIGNALS:
-    void annotationContextMenuAboutToShow(KTextEditor::View *view, QMenu *menu, int line) Q_DECL_OVERRIDE;
-    void annotationActivated(KTextEditor::View *view, int line) Q_DECL_OVERRIDE;
+    void annotationContextMenuAboutToShow(KTextEditor::View *view, QMenu *menu, int line) override;
+    void annotationActivated(KTextEditor::View *view, int line) override;
     // KF6: fix View -> KTextEditor::View
-    void annotationBorderVisibilityChanged(View *view, bool visible) Q_DECL_OVERRIDE;
+    void annotationBorderVisibilityChanged(View *view, bool visible) override;
 
     void navigateLeft();
     void navigateRight();
@@ -418,35 +461,80 @@ public Q_SLOTS:
     void goToPreviousEditingPosition();
 
     /**
-     * Sets the cursor to the next editing position in this document
+     * Sets the cursor to the next editing position in this document.
      */
     void goToNextEditingPosition();
 
     /**
-      Uppercases selected text, or an alphabetic character next to the cursor.
-    */
+     * Uppercases selected text, or an alphabetic character next to the cursor.
+     */
     void uppercase();
+
     /**
-      Lowercases selected text, or an alphabetic character next to the cursor.
-    */
+     * Lowercases selected text, or an alphabetic character next to the cursor.
+     */
     void lowercase();
+
     /**
-      Capitalizes the selection (makes each word start with an uppercase) or
-      the word under the cursor.
-    */
+     * Capitalizes the selection (makes each word start with an uppercase) or
+     * the word under the cursor.
+     */
     void capitalize();
+
     /**
-      Joins lines touched by the selection
-    */
+     * Joins lines touched by the selection.
+     */
     void joinLines();
 
-    // Note - the following functions simply forward to KateViewInternal
+    /**
+     * Performs a line break (insert a new line char) at current cursor position
+     * and indent the new line.
+     *
+     * Most work is done by @c KTextEditor::DocumentPrivate::newLine and
+     * @c KateAutoIndent::userTypedChar
+     * @see KTextEditor::DocumentPrivate::newLine, KateAutoIndent
+     */
     void keyReturn();
+
+    /**
+     * Performs a line break (insert a new line char) at current cursor position
+     * but keep all leading non word char as indent for the new line.
+     */
     void smartNewline();
+
+    /**
+     * Insert a tabulator char at current cursor position.
+     */
     void backspace();
+
+    /**
+     * Remove the word left from the current cursor position including all leading
+     * space.
+     * @see KateViewInternal::wordPrev
+     */
     void deleteWordLeft();
+
+    /**
+     * Remove the current selection. When nothing is selected the char right
+     * from the current cursor position is removed.
+     * @see KTextEditor::DocumentPrivate::del
+     */
     void keyDelete();
+
+    /**
+     * When the char right from the current cursor position is a space is all
+     * space to the right removed. Otherwise is the word to the right including
+     * all trialling space removed.
+     * @see KateViewInternal::wordNext
+     */
     void deleteWordRight();
+
+    /**
+     * Transpose the characters left and right from the current cursor position
+     * and move the cursor one position to the right. If the char right to the
+     * current cursor position is a new line char, nothing is done.
+     * @see KTextEditor::DocumentPrivate::transpose
+     */
     void transpose();
     void cursorLeft();
     void shiftCursorLeft();
@@ -482,6 +570,9 @@ public Q_SLOTS:
     void shiftToMatchingBracket();
     void toPrevModifiedLine();
     void toNextModifiedLine();
+    /**
+     * Insert a tabulator char at current cursor position.
+     */
     void insertTab();
 
     void gotoLine();
@@ -501,7 +592,7 @@ public:
      * \param flags additional flags
      * \see writeSessionConfig()
      */
-    void readSessionConfig(const KConfigGroup &config, const QSet<QString> &flags = QSet<QString>()) Q_DECL_OVERRIDE;
+    void readSessionConfig(const KConfigGroup &config, const QSet<QString> &flags = QSet<QString>()) override;
 
     /**
      * Write session settings to the \p config.
@@ -511,7 +602,7 @@ public:
      * \param flags additional flags
      * \see readSessionConfig()
      */
-    void writeSessionConfig(KConfigGroup &config, const QSet<QString> &flags = QSet<QString>()) Q_DECL_OVERRIDE;
+    void writeSessionConfig(KConfigGroup &config, const QSet<QString> &flags = QSet<QString>()) override;
 
 public Q_SLOTS:
     void setEol(int eol);
@@ -607,18 +698,20 @@ public:
     void exportHtmlToFile(const QString &file);
 
 private Q_SLOTS:
+    void slotDocumentReloaded();
+    void slotDocumentAboutToReload();
     void slotGotFocus();
     void slotLostFocus();
     void slotSaveCanceled(const QString &error);
     void slotConfigDialog();
-    void exportHtmlToClipboard ();
-    void exportHtmlToFile ();
+    void exportHtmlToClipboard();
+    void exportHtmlToFile();
 
 public Q_SLOTS:
     void slotFoldToplevelNodes();
     void slotExpandToplevelNodes();
-    void slotCollapseLocal();
-    void slotExpandLocal();
+    void slotToggleFolding();
+    void slotToggleFoldingsInRange();
 
 private:
     void setupLayout();
@@ -627,10 +720,11 @@ private:
     void setupEditActions();
     void setupCodeFolding();
 
-    QList<QAction *>        m_editActions;
+    QList<QAction *>       m_editActions;
     QAction               *m_editUndo;
     QAction               *m_editRedo;
     QAction               *m_pasteMenu;
+    bool                   m_gotoBottomAfterReload;
     KToggleAction         *m_toggleFoldingMarkers;
     KToggleAction         *m_toggleIconBar;
     KToggleAction         *m_toggleLineNumbers;
@@ -746,8 +840,8 @@ Q_SIGNALS:
     void displayRangeChanged(KTextEditor::ViewPrivate *view);
 
 protected:
-    bool event(QEvent *e) Q_DECL_OVERRIDE;
-    void paintEvent(QPaintEvent *e) Q_DECL_OVERRIDE;
+    bool event(QEvent *e) override;
+    void paintEvent(QPaintEvent *e) override;
 
     KToggleAction               *m_toggleOnTheFlySpellCheck;
     KateSpellingMenu *m_spellingMenu;
@@ -811,6 +905,12 @@ Q_SIGNALS:
      */
     void delayedUpdateOfView();
 
+    /**
+     * Emitted whenever the caret enter or leave a range.
+     * ATM only used by KateStatusBar to update the dict button
+     */
+    void caretChangedRange(KTextEditor::View *);
+
 public:
     /**
      * set of ranges which had the mouse inside last time, used for rendering
@@ -897,8 +997,8 @@ private:
     // KTextEditor::PrintInterface
     //
 public Q_SLOTS:
-    bool print() Q_DECL_OVERRIDE;
-    void printPreview() Q_DECL_OVERRIDE;
+    bool print() override;
+    void printPreview() override;
 
 public:
     /**
@@ -942,7 +1042,7 @@ private:
     KateViewEncodingAction *m_encodingAction;
 
     /**
-     * the mode selection menu, used by view + status bar
+     * the mode selection menu, used by view
      */
     KateModeMenu *m_modeAction;
 
@@ -955,14 +1055,14 @@ public:
     /**
      * Returns the attribute for the default style \p defaultStyle.
      */
-    Attribute::Ptr defaultStyleAttribute(DefaultStyle defaultStyle) const Q_DECL_OVERRIDE;
+    Attribute::Ptr defaultStyleAttribute(DefaultStyle defaultStyle) const override;
 
     /**
      * Get the list of AttributeBlocks for a given \p line in the document.
      *
      * \return list of AttributeBlocks for given \p line.
      */
-    QList<KTextEditor::AttributeBlock> lineAttributes(int line) Q_DECL_OVERRIDE;
+    QList<KTextEditor::AttributeBlock> lineAttributes(int line) override;
 
 private:
     // remember folding state to prevent refolding the first line if it was manually unfolded,
